/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.multiblock;

import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import techreborn.init.TRContent;

public class MultiblockChecker {

	public static final class_2338 ZERO_OFFSET = class_2338.field_10980;

	public static final String STANDARD_CASING = "standard";
	public static final String REINFORCED_CASING = "reinforced";
	public static final String ADVANCED_CASING = "advanced";
	public static final String CASING_ANY = "any";

	private final class_1937 world;
	private final class_2338 downCenter;

	public MultiblockChecker(class_1937 world, class_2338 downCenter) {
		this.world = world;
		this.downCenter = downCenter;
	}

	// TODO: make thid not so ugly
	public boolean checkCasing(int offX, int offY, int offZ, String type) {
		class_2248 block = getBlock(offX, offY, offZ).method_11614();
		if (block == TRContent.MachineBlocks.BASIC.getCasing()|| block == TRContent.MachineBlocks.ADVANCED.getCasing() || block == TRContent.MachineBlocks.INDUSTRIAL.getCasing() ) {
			if (type == MultiblockChecker.CASING_ANY) {
				return true;
			} else if ( type == "standard" && block ==  TRContent.MachineBlocks.BASIC.getCasing()) {
				return true;
			}
			else if (type == "reinforced" && block ==  TRContent.MachineBlocks.ADVANCED.getCasing()) {
				return true;
			}
			else if (type == "advanced" && block ==  TRContent.MachineBlocks.INDUSTRIAL.getCasing()) {
				return true;
			}
		}
		return false;
	}

	public boolean checkAir(int offX, int offY, int offZ) {
		class_2338 pos = downCenter.method_10069(offX, offY, offZ);
		return world.method_22347(pos);
	}

	public class_2680 getBlock(int offX, int offY, int offZ) {
		class_2338 pos = downCenter.method_10069(offX, offY, offZ);
		return world.method_8320(pos);
	}

	public boolean checkRectY(int sizeX, int sizeZ, String casingType, class_2338 offset) {
		for (int x = -sizeX; x <= sizeX; x++) {
			for (int z = -sizeZ; z <= sizeZ; z++) {
				if (!checkCasing(x + offset.method_10263(), offset.method_10264(), z + offset.method_10260(), casingType))
					return false;
			}
		}
		return true;
	}

	public boolean checkRectZ(int sizeX, int sizeY, String casingType, class_2338 offset) {
		for (int x = -sizeX; x <= sizeX; x++) {
			for (int y = -sizeY; y <= sizeY; y++) {
				if (!checkCasing(x + offset.method_10263(), y + offset.method_10264(), offset.method_10260(), casingType))
					return false;
			}
		}
		return true;
	}

	public boolean checkRectX(int sizeZ, int sizeY, String casingType, class_2338 offset) {
		for (int z = -sizeZ; z <= sizeZ; z++) {
			for (int y = -sizeY; y <= sizeY; y++) {
				if (!checkCasing(offset.method_10263(), y + offset.method_10264(), z + offset.method_10260(), casingType))
					return false;
			}
		}
		return true;
	}

	public boolean checkRingY(int sizeX, int sizeZ, String casingType, class_2338 offset) {
		for (int x = -sizeX; x <= sizeX; x++) {
			for (int z = -sizeZ; z <= sizeZ; z++) {
				if ((x == sizeX || x == -sizeX) || (z == sizeZ || z == -sizeZ)) {
					if (!checkCasing(x + offset.method_10263(), offset.method_10264(), z + offset.method_10260(), casingType))
						return false;
				}
			}
		}
		return true;
	}

	public boolean checkRingYHollow(int sizeX, int sizeZ, String casingType, class_2338 offset) {
		for (int x = -sizeX; x <= sizeX; x++) {
			for (int z = -sizeZ; z <= sizeZ; z++) {
				if ((x == sizeX || x == -sizeX) || (z == sizeZ || z == -sizeZ)) {
					if (!checkCasing(x + offset.method_10263(), offset.method_10264(), z + offset.method_10260(), casingType))
						return false;
				} else if (!checkAir(x + offset.method_10263(), offset.method_10264(), z + offset.method_10260()))
					return false;
			}
		}
		return true;
	}

}
