/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier1;

import org.apache.commons.lang3.tuple.Pair;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.recipes.ExtendedRecipeRemainder;
import reborncore.common.util.IInventoryAccess;
import reborncore.common.util.ItemUtils;
import reborncore.common.util.RebornInventory;
import techreborn.config.TechRebornConfig;
import techreborn.init.ModSounds;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

import javax.annotation.Nullable;
import net.minecraft.class_1657;
import net.minecraft.class_1703;
import net.minecraft.class_1715;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_2350;
import net.minecraft.class_2371;
import net.minecraft.class_2487;
import net.minecraft.class_3419;
import net.minecraft.class_3955;
import net.minecraft.class_3956;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * Created by modmuss50 on 20/06/2017.
 */
public class AutoCraftingTableBlockEntity extends PowerAcceptorBlockEntity
		implements IToolDrop, InventoryProvider, IContainerProvider {

	public RebornInventory<AutoCraftingTableBlockEntity> inventory = new RebornInventory<>(11, "AutoCraftingTableBlockEntity", 64, this);
	public int progress;
	public int maxProgress = 120;
	public int euTick = 10;
	public int balanceSlot = 0;

	class_1715 inventoryCrafting = null;
	class_3955 lastCustomRecipe = null;
	class_3955 lastRecipe = null;

	public boolean locked = true;

	public AutoCraftingTableBlockEntity() {
		super(TRBlockEntities.AUTO_CRAFTING_TABLE);
	}

	@Nullable
	public class_3955 getCurrentRecipe() {
		class_1715 crafting = getCraftingInventory();
		if (!crafting.method_5442()) {
			if (lastRecipe != null) {
				if (lastRecipe.method_8115(crafting, field_11863)) {
					return lastRecipe;
				}
			}
			Optional<class_3955> testRecipe = field_11863.method_8433().method_8132(class_3956.field_17545, crafting, field_11863);
			if (testRecipe.isPresent()) {
				lastRecipe = testRecipe.get();
				return lastRecipe;
			}
		}
		return null;
	}

	public class_1715 getCraftingInventory() {
		if (inventoryCrafting == null) {
			inventoryCrafting = new class_1715(new class_1703(null, -1) {
				@Override
				public boolean method_7597(class_1657 playerIn) {
					return false;
				}
			}, 3, 3);
		}
		for (int i = 0; i < 9; i++) {
			inventoryCrafting.method_5447(i, inventory.method_5438(i));
		}
		return inventoryCrafting;
	}

	public boolean canMake(class_3955 recipe) {
		if (recipe != null) {
			boolean missingOutput = false;
			int[] stacksInSlots = new int[9];
			for (int i = 0; i < 9; i++) {
				stacksInSlots[i] = inventory.method_5438(i).method_7947();
			}

			class_2371<class_1856> ingredients = recipe.method_8117();
			List<Integer> checkedSlots = new ArrayList<>();
			for (class_1856 ingredient : ingredients) {
				if (ingredient != class_1856.field_9017) {
					boolean foundIngredient = false;
					for (int i = 0; i < 9; i++) {
						if(checkedSlots.contains(i)) {
							continue;
						}
						class_1799 stack = inventory.method_5438(i);
						int requiredSize = locked ? 1 : 0;
						if (stack.method_7914() == 1) {
							requiredSize = 0;
						}
						if (stacksInSlots[i] > requiredSize) {
							if (ingredient.method_8093(stack)) {
								foundIngredient = true;
								checkedSlots.add(i);
								stacksInSlots[i]--;
								break;
							}
						}
					}
					if (!foundIngredient) {
						missingOutput = true;
					}
				}
			}
			if (!missingOutput) {
				if (hasOutputSpace(recipe.method_8110(), 9)) {
					return true;
				}
			}
			return false;
		}
		return false;
	}

	boolean hasRoomForExtraItem(class_1799 stack) {
		class_1799 extraOutputSlot = inventory.method_5438(10);
		if (extraOutputSlot.method_7960()) {
			return true;
		}
		return hasOutputSpace(stack, 10);
	}

	public boolean hasOutputSpace(class_1799 output, int slot) {
		class_1799 stack = inventory.method_5438(slot);
		if (stack.method_7960()) {
			return true;
		}
		if (ItemUtils.isItemEqual(stack, output, true, true)) {
			if (stack.method_7914() > stack.method_7947() + output.method_7947()) {
				return true;
			}
		}
		return false;
	}

	public boolean make(class_3955 recipe) {
		if (recipe == null || !canMake(recipe)) {
			return false;
		}
		for (int i = 0; i < recipe.method_8117().size(); i++) {
			class_2371<class_1856> ingredients = recipe.method_8117();
			class_1856 ingredient = ingredients.get(i);
			// Looks for the best slot to take it from
			class_1799 bestSlot = inventory.method_5438(i);
			if (ingredient.method_8093(bestSlot)) {
				class_1799 remainderStack = getRemainderItem(bestSlot);
				if(remainderStack.method_7960()) {
					bestSlot.method_7934(1);
				} else {
					inventory.method_5447(i, remainderStack);
				}

			} else {
				for (int j = 0; j < 9; j++) {
					class_1799 stack = inventory.method_5438(j);
					if (ingredient.method_8093(stack)) {
						class_1799 remainderStack = getRemainderItem(stack);
						if(remainderStack.method_7960()) {
							stack.method_7934(1);
						} else {
							inventory.method_5447(j, remainderStack);
						}
						break;
					}
				}
			}
		}
		class_1799 output = inventory.method_5438(9);
		class_1799 outputStack = recipe.method_8116(getCraftingInventory());
		if (output.method_7960()) {
			inventory.method_5447(9, outputStack.method_7972());
		} else {
			output.method_7933(recipe.method_8110().method_7947());
		}
		return true;
	}

	private class_1799 getRemainderItem(class_1799 stack) {
		if(stack.method_7909() instanceof ExtendedRecipeRemainder) {
			return ((ExtendedRecipeRemainder) stack.method_7909()).getRemainderStack(stack);

		} else if (stack.method_7909().method_7857()) {
			return new class_1799(stack.method_7909().method_7858());
		}
		return class_1799.field_8037;
	}

	public int getProgress() {
		return progress;
	}

	public void setProgress(int progress) {
		this.progress = progress;
	}

	public int getMaxProgress() {
		if (maxProgress == 0) {
			maxProgress = 1;
		}
		return maxProgress;
	}

	public void setMaxProgress(int maxProgress) {
		this.maxProgress = maxProgress;
	}

	// TilePowerAcceptor
	@Override
	public void method_16896() {
		super.method_16896();
		if (field_11863.field_9236) {
			return;
		}
		class_3955 recipe = getCurrentRecipe();
		if (recipe != null) {
			Optional<class_1715> balanceResult = balanceRecipe(getCraftingInventory());
			balanceResult.ifPresent(craftingInventory -> inventoryCrafting = craftingInventory);

			if (progress >= maxProgress) {
				if (make(recipe)) {
					progress = 0;
				}
			} else {
				if (canMake(recipe)) {
					if (canUseEnergy(euTick)) {
						progress++;
						if (progress == 1) {
							field_11863.method_8465(null, field_11867.method_10263(), field_11867.method_10264(), field_11867.method_10260(), ModSounds.AUTO_CRAFTING,
									class_3419.field_15245, 0.3F, 0.8F);
						}
						useEnergy(euTick);
					}
				} else {
					progress = 0;
				}
			}
		}
		if (recipe == null) {
			progress = 0;
		}
	}

	public Optional<class_1715> balanceRecipe(class_1715 craftCache) {
		class_3955 currentRecipe = getCurrentRecipe();
		if (currentRecipe == null) {
			return Optional.empty();
		}
		if (field_11863.field_9236) {
			return Optional.empty();
		}
		if (craftCache.method_5442()) {
			return Optional.empty();
		}
		balanceSlot++;
		if (balanceSlot > craftCache.method_5439()) {
			balanceSlot = 0;
		}
		//Find the best slot for each item in a recipe, and move it if needed
		class_1799 sourceStack = inventory.method_5438(balanceSlot);
		if (sourceStack.method_7960()) {
			return Optional.empty();
		}
		List<Integer> possibleSlots = new ArrayList<>();
		for (int s = 0; s < currentRecipe.method_8117().size(); s++) {
			for (int i = 0; i < 9; i++) {
				if(possibleSlots.contains(i)) {
					continue;
				}
				class_1799 stackInSlot = inventory.method_5438(i);
				class_1856 ingredient = currentRecipe.method_8117().get(s);
				if (ingredient != class_1856.field_9017 && ingredient.method_8093(sourceStack)) {
					if (stackInSlot.method_7909() == sourceStack.method_7909()) {
						possibleSlots.add(i);
						break;
					}
				}
			}

		}

		if(!possibleSlots.isEmpty()){
			int totalItems =  possibleSlots.stream()
					.mapToInt(value -> inventory.method_5438(value).method_7947()).sum();
			int slots = possibleSlots.size();

			//This makes an array of ints with the best possible slot EnvTyperibution
			int[] split = new int[slots];
			int remainder = totalItems % slots;
			Arrays.fill(split, totalItems / slots);
			while (remainder > 0){
				for (int i = 0; i < split.length; i++) {
					if(remainder > 0){
						split[i] +=1;
						remainder --;
					}
				}
			}

			List<Integer> slotEnvTyperubution = possibleSlots.stream()
					.mapToInt(value -> inventory.method_5438(value).method_7947())
					.boxed().collect(Collectors.toList());

			boolean needsBalance = false;
			for (int i = 0; i < split.length; i++) {
				int required = split[i];
				if(slotEnvTyperubution.contains(required)){
					//We need to remove the int, not at the int, this seems to work around that
					slotEnvTyperubution.remove(new Integer(required));
				} else {
					needsBalance = true;
				}
			}
			if (!needsBalance) {
				return Optional.empty();
			}
		} else {
			return Optional.empty();
		}

		//Slot, count
		Pair<Integer, Integer> bestSlot = null;
		for (Integer slot : possibleSlots) {
			class_1799 slotStack = inventory.method_5438(slot);
			if (slotStack.method_7960()) {
				bestSlot = Pair.of(slot, 0);
			}
			if (bestSlot == null) {
				bestSlot = Pair.of(slot, slotStack.method_7947());
			} else if (bestSlot.getRight() >= slotStack.method_7947()) {
				bestSlot = Pair.of(slot, slotStack.method_7947());
			}
		}
		if (bestSlot == null
				|| bestSlot.getLeft() == balanceSlot
				|| bestSlot.getRight() == sourceStack.method_7947()
				|| inventory.method_5438(bestSlot.getLeft()).method_7960()
				|| !ItemUtils.isItemEqual(sourceStack, inventory.method_5438(bestSlot.getLeft()), true, true)) {
			return Optional.empty();
		}
		sourceStack.method_7934(1);
		inventory.method_5438(bestSlot.getLeft()).method_7933(1);
		inventory.setChanged();

		return Optional.of(getCraftingInventory());
	}

	// Easyest way to sync back to the client
	public int getLockedInt() {
		return locked ? 1 : 0;
	}

	public void setLockedInt(int lockedInt) {
		locked = lockedInt == 1;
	}

	@Override
	public double getBaseMaxPower() {
		return TechRebornConfig.autoCraftingTableMaxEnergy;
	}

	@Override
	public double getBaseMaxOutput() {
		return 0;
	}

	@Override
	public double getBaseMaxInput() {
		return TechRebornConfig.autoCraftingTableMaxInput;
	}

	@Override
	public boolean canAcceptEnergy(class_2350 enumFacing) {
		return true;
	}

	@Override
	public boolean canProvideEnergy(class_2350 enumFacing) {
		return false;
	}

	@Override
	public class_2487 method_11007(class_2487 tag) {
		tag.method_10556("locked", locked);
		return super.method_11007(tag);
	}

	@Override
	public void method_11014(class_2487 tag) {
		if (tag.method_10545("locked")) {
			locked = tag.method_10577("locked");
		}
		super.method_11014(tag);
	}

	// TileMachineBase
	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	// This machine doesnt have a facing
	@Override
	public class_2350 getFacingEnum() {
		return class_2350.field_11043;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 playerIn) {
		return TRContent.Machine.AUTO_CRAFTING_TABLE.getStack();
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory<AutoCraftingTableBlockEntity> getInventory() {
		return inventory;
	}

	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, class_1657 player) {
		return new ContainerBuilder("autocraftingtable").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this).slot(0, 28, 25).slot(1, 46, 25).slot(2, 64, 25).slot(3, 28, 43).slot(4, 46, 43)
				.slot(5, 64, 43).slot(6, 28, 61).slot(7, 46, 61).slot(8, 64, 61).outputSlot(9, 145, 42)
				.outputSlot(10, 145, 70).syncEnergyValue().sync(this::getProgress, this::setProgress)
				.sync(this::getMaxProgress, this::setMaxProgress)
				.sync(this::getLockedInt, this::setLockedInt).addInventory().create(this, syncID);
	}

	@Override
	public boolean hasSlotConfig() {
		return true;
	}
}
