/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier1;

import net.minecraft.block.*;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2211;
import net.minecraft.class_2248;
import net.minecraft.class_2256;
import net.minecraft.class_2261;
import net.minecraft.class_2266;
import net.minecraft.class_2282;
import net.minecraft.class_2302;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2421;
import net.minecraft.class_2511;
import net.minecraft.class_2523;
import net.minecraft.class_2680;
import net.minecraft.class_2758;
import net.minecraft.class_3218;
import net.minecraft.class_3830;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.util.ItemUtils;
import reborncore.common.util.RebornInventory;
import techreborn.blocks.lighting.BlockLamp;
import techreborn.blocks.misc.BlockRubberLog;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

import java.util.Collections;
import java.util.List;

public class GreenhouseControllerBlockEntity extends PowerAcceptorBlockEntity
		implements IToolDrop, InventoryProvider, IContainerProvider {
	
	private final RebornInventory<GreenhouseControllerBlockEntity> inventory = new RebornInventory<>(7, "GreenhouseControllerBlockEntity", 64, this);
	private class_2338 multiblockCenter;
	private int ticksToNextMultiblockCheck = 0;
	private boolean growthBoost = false;
	
	public GreenhouseControllerBlockEntity() {
		super(TRBlockEntities.GREENHOUSE_CONTROLLER);
	}
	
	public boolean getMultiBlock() {
		if (multiblockCenter == null || field_11863 == null) {
			return false;
		}
		for (int i = -1; i <= 1; i++) {
			for (int j = -1; j <= 1; j++) {
				class_2680 block = field_11863.method_8320(multiblockCenter.method_10069(i * 3, 3, i * 3));
				if (!(block.method_11614() instanceof BlockLamp) || !BlockLamp.isActive(block)) {
					return false;
				}
			}
		}
		return true;
	}
	
	@Override
	public void method_16896() {
		if (multiblockCenter == null) {
			multiblockCenter = field_11867.method_10079(getFacing().method_10153(), 5);
		}
		charge(6);
		super.method_16896();
		
		if (field_11863.field_9236) {
			return;
		}
		
		if (useEnergy(getEuPerTick(TechRebornConfig.greenhouseControllerEnergyPerTick)) != getEuPerTick(TechRebornConfig.greenhouseControllerEnergyPerTick)) {
			return;
		}
		
		if (--ticksToNextMultiblockCheck < 0) {
			growthBoost = getMultiBlock();
			ticksToNextMultiblockCheck = 200;
		}
		
		if (field_11863.method_8510() % 20 == 0) {
			double cyclesLimit = getSpeedMultiplier() * 4 + 1;
			while (cyclesLimit-- > 0) {
				workCycle();
			}
		}
		
	}
	
	private void workCycle() {
		class_2338 blockPos = multiblockCenter.method_10069(field_11863.field_9229.nextInt(9) - 4, 0, field_11863.field_9229.nextInt(9) - 4);
		class_2680 blockState = field_11863.method_8320(blockPos);
		class_2248 block = blockState.method_11614();
		
		if (growthBoost) {
			if (block instanceof class_2256
					|| block instanceof class_2261
					|| block instanceof class_2523
					|| block instanceof class_2266
			) {
				if (canUseEnergy(TechRebornConfig.greenhouseControllerEnergyPerBonemeal)) {
					useEnergy(TechRebornConfig.greenhouseControllerEnergyPerBonemeal);
					blockState.method_11585((class_3218) field_11863, blockPos, field_11863.field_9229);
				}
			}
		}
		
		if (block instanceof class_2302) {
			processAgedCrop(blockState, blockPos, ((class_2302) block).method_9824(), ((class_2302) block).method_9827(), 0);
		} else if (block instanceof class_2421) {
			processAgedCrop(blockState, blockPos, class_2421.field_11306, 3, 0);
		} else if (block instanceof class_3830) {
			processAgedCrop(blockState, blockPos, class_3830.field_17000, 3, 1);
		} else if (block instanceof class_2282) {
			processAgedCrop(blockState, blockPos, class_2282.field_10779, 2, 0);
		} else if (block instanceof class_2511) {
			if (tryHarvestBlock(blockState, blockPos)) {
				field_11863.method_22352(blockPos, false);
			}
		} else if (block instanceof class_2523
				|| block instanceof class_2266
				|| block instanceof class_2211
		) {
			// If we can break bottom block we should at least remove all of them up to top so they don't break automatically
			boolean breakBlocks = false;
			for (int y = 1; (blockState = field_11863.method_8320(blockPos.method_10086(y))).method_11614() == block; y++) {
				if (y == 1) {
					breakBlocks = tryHarvestBlock(blockState, blockPos.method_10086(y));
				} else {
					tryHarvestBlock(blockState, blockPos.method_10086(y));
				}
				if (breakBlocks) field_11863.method_22352(blockPos.method_10086(y), false);
			}
		} else if (block instanceof BlockRubberLog) {
			for (int y = 1; (blockState = field_11863.method_8320(blockPos.method_10086(y))).method_11614() == block && y < 10; y++) {
				if (blockState.method_11654(BlockRubberLog.HAS_SAP)
						&& canUseEnergy(TechRebornConfig.greenhouseControllerEnergyPerHarvest)
						&& insertIntoInv(Collections.singletonList(TRContent.Parts.SAP.getStack()))
				) {
					useEnergy(TechRebornConfig.greenhouseControllerEnergyPerHarvest);
					field_11863.method_8501(blockPos.method_10086(y), blockState.method_11657(BlockRubberLog.HAS_SAP, false).method_11657(BlockRubberLog.SAP_SIDE, class_2350.method_10139(0)));
				}
			}
		}
		
	}
	
	private void processAgedCrop(class_2680 blockState, class_2338 blockPos, class_2758 ageProperty, int maxAge, int newAge) {
		if (blockState.method_11654(ageProperty) >= maxAge) {
			if (tryHarvestBlock(blockState, blockPos)) {
				field_11863.method_8652(blockPos, blockState.method_11657(ageProperty, newAge), 2);
			}
		}
	}
	
	private boolean tryHarvestBlock(class_2680 blockState, class_2338 blockPos) {
		if (canUseEnergy(TechRebornConfig.greenhouseControllerEnergyPerHarvest)
				&& insertIntoInv(class_2248.method_9562(blockState, (class_3218) field_11863, blockPos, null))) {
			useEnergy(TechRebornConfig.greenhouseControllerEnergyPerHarvest);
			return true;
		}
		return false;
	}
	
	private boolean insertIntoInv(List<class_1799> stacks) {
		boolean result = false;
		for (class_1799 stack : stacks) {
			for (int i = 0; i < 6; i++) {
				if (insertIntoInv(i, stack)) result = true;
				if (stack.method_7960()) break;
			}
		}
		return result;
	}
	
	private boolean insertIntoInv(int slot, class_1799 stack) {
		class_1799 targetStack = inventory.method_5438(slot);
		if (targetStack.method_7960()) {
			inventory.method_5447(slot, stack.method_7972());
			stack.method_7934(stack.method_7947());
			return true;
		} else {
			if (ItemUtils.isItemEqual(stack, targetStack, true, false)) {
				int freeStackSpace = targetStack.method_7914() - targetStack.method_7947();
				if (freeStackSpace > 0) {
					int transferAmount = Math.min(freeStackSpace, stack.method_7947());
					targetStack.method_7933(transferAmount);
					stack.method_7934(transferAmount);
					return true;
				}
			}
		}
		return false;
	}
	
	@Override
	public boolean canAcceptEnergy(class_2350 direction) {
		return true;
	}
	
	@Override
	public boolean canProvideEnergy(class_2350 direction) {
		return false;
	}
	
	@Override
	public double getBaseMaxPower() {
		return TechRebornConfig.greenhouseControllerMaxEnergy;
	}
	
	@Override
	public double getBaseMaxOutput() {
		return 0;
	}
	
	@Override
	public double getBaseMaxInput() {
		return TechRebornConfig.greenhouseControllerMaxInput;
	}
	
	@Override
	public class_1799 getToolDrop(class_1657 entityPlayer) {
		return TRContent.Machine.GREENHOUSE_CONTROLLER.getStack();
	}
	
	@Override
	public RebornInventory<GreenhouseControllerBlockEntity> getInventory() {
		return this.inventory;
	}
	
	@Override
	public boolean canBeUpgraded() {
		return true;
	}
	
	@Override
	public BuiltContainer createContainer(int syncID, class_1657 player) {
		return new ContainerBuilder("greenhousecontroller").player(player.field_7514).inventory().hotbar().addInventory()
				.blockEntity(this)
				.outputSlot(0, 30, 22).outputSlot(1, 48, 22)
				.outputSlot(2, 30, 40).outputSlot(3, 48, 40)
				.outputSlot(4, 30, 58).outputSlot(5, 48, 58)
				.energySlot(6, 8, 72).syncEnergyValue()
				.addInventory().create(this, syncID);
	}
	
}
