/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.tier1;

import org.apache.commons.lang3.tuple.Pair;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blocks.BlockMachineBase;
import reborncore.common.powerSystem.PowerAcceptorBlockEntity;
import reborncore.common.util.ItemUtils;
import reborncore.common.util.RebornInventory;
import techreborn.api.recipe.recipes.RollingMachineRecipe;
import techreborn.config.TechRebornConfig;
import techreborn.init.ModRecipes;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

import javax.annotation.Nonnull;
import net.minecraft.class_1657;
import net.minecraft.class_1703;
import net.minecraft.class_1715;
import net.minecraft.class_1799;
import net.minecraft.class_1856;
import net.minecraft.class_1937;
import net.minecraft.class_2350;
import net.minecraft.class_2487;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

//TODO add tick and power bars.

public class RollingMachineBlockEntity extends PowerAcceptorBlockEntity
	implements IToolDrop, InventoryProvider, IContainerProvider {

	public int[] craftingSlots = new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8 };
	private class_1715 craftCache;
	public RebornInventory<RollingMachineBlockEntity> inventory = new RebornInventory<>(12, "RollingMachineBlockEntity", 64, this);
	public boolean isRunning;
	public int tickTime;
	@Nonnull
	public class_1799 currentRecipeOutput;
	public RollingMachineRecipe currentRecipe;
	private int outputSlot;
	public boolean locked = false;
	public int balanceSlot = 0;

	public RollingMachineBlockEntity() {
		super(TRBlockEntities.ROLLING_MACHINE);
		outputSlot = 9;
	}

	@Override
	public double getBaseMaxPower() {
		return TechRebornConfig.rollingMachineMaxEnergy;
	}

	@Override
	public boolean canAcceptEnergy(final class_2350 direction) {
		return true;
	}

	@Override
	public boolean canProvideEnergy(final class_2350 direction) {
		return false;
	}

	@Override
	public double getBaseMaxOutput() {
		return 0;
	}

	@Override
	public double getBaseMaxInput() {
		return TechRebornConfig.rollingMachineMaxInput;
	}

	@Override
	public void method_16896() {
		super.method_16896();
		if (field_11863.field_9236) {
			return;
		}
		charge(10);

		class_1715 craftMatrix = getCraftingMatrix();
		currentRecipe = findMatchingRecipe(craftMatrix, field_11863);
		if (currentRecipe != null) {
			setIsActive(true);
			if (field_11863.method_8510() % 2 == 0) {
				Optional<class_1715> balanceResult = balanceRecipe(craftMatrix);
				if (balanceResult.isPresent()) {
					craftMatrix = balanceResult.get();
				}
			}
			currentRecipeOutput = currentRecipe.method_8116(craftMatrix);
		} else {
			currentRecipeOutput = class_1799.field_8037;
		}

		if (!currentRecipeOutput.method_7960() && canMake(craftMatrix)) {
			if (tickTime >= Math.max((int) (TechRebornConfig.rollingMachineRunTime * (1.0 - getSpeedMultiplier())), 1)) {
				currentRecipeOutput = findMatchingRecipeOutput(craftMatrix, field_11863);
				if (!currentRecipeOutput.method_7960()) {
					boolean hasCrafted = false;
					if (inventory.method_5438(outputSlot).method_7960()) {
						inventory.method_5447(outputSlot, currentRecipeOutput.method_7972());
						tickTime = 0;
						hasCrafted = true;
					} else {
						if (inventory.method_5438(outputSlot).method_7947()
								+ currentRecipeOutput.method_7947() <= currentRecipeOutput.method_7914()) {
							final class_1799 stack = inventory.method_5438(outputSlot);
							stack.method_7939(stack.method_7947() + currentRecipeOutput.method_7947());
							inventory.method_5447(outputSlot, stack);
							tickTime = 0;
							hasCrafted = true;
						} else {
							setIsActive(false);
						}
					}
					if (hasCrafted) {
						for (int i = 0; i < craftMatrix.method_5439(); i++) {
							inventory.shrinkSlot(i, 1);
						}
						currentRecipeOutput = class_1799.field_8037;
						currentRecipe = null;
					}
				}
			}
		} else {
			tickTime = 0;
		}
		if (!currentRecipeOutput.method_7960()) {
			if (canUseEnergy(getEuPerTick(TechRebornConfig.rollingMachineEnergyPerTick))
					&& tickTime < Math.max((int) (TechRebornConfig.rollingMachineRunTime * (1.0 - getSpeedMultiplier())), 1)
					&& canMake(craftMatrix)) {
				useEnergy(getEuPerTick(TechRebornConfig.rollingMachineEnergyPerTick));
				tickTime++;
			} else {
				setIsActive(false);
			}
		}
		if (currentRecipeOutput.method_7960()) {
			tickTime = 0;
			currentRecipe = null;
			setIsActive(canMake(getCraftingMatrix()));
		}
	}

	public void setIsActive(boolean active) {
		if (active == isRunning){
			return;
		}
		isRunning = active;
		if (this.method_10997().method_8320(this.method_11016()).method_11614() instanceof BlockMachineBase) {
			BlockMachineBase blockMachineBase = (BlockMachineBase)this.method_10997().method_8320(this.method_11016()).method_11614();
			blockMachineBase.setActive(active, this.method_10997(), this.method_11016());
		}
		this.method_10997().method_8413(this.method_11016(), this.method_10997().method_8320(this.method_11016()), this.method_10997().method_8320(this.method_11016()), 3);
	}

	public Optional<class_1715> balanceRecipe(class_1715 craftCache) {
		if (currentRecipe == null) {
			return Optional.empty();
		}
		if (field_11863.field_9236) {
			return Optional.empty();
		}
		if (!locked) {
			return Optional.empty();
		}
		if (craftCache.method_5442()) {
			return Optional.empty();
		}
		balanceSlot++;
		if (balanceSlot > craftCache.method_5439()) {
			balanceSlot = 0;
		}
		//Find the best slot for each item in a recipe, and move it if needed
		class_1799 sourceStack = inventory.method_5438(balanceSlot);
		if (sourceStack.method_7960()) {
			return Optional.empty();
		}
		List<Integer> possibleSlots = new ArrayList<>();
		for (int s = 0; s < currentRecipe.method_8117().size(); s++) {
			class_1799 stackInSlot = inventory.method_5438(s);
			class_1856 ingredient = (class_1856) currentRecipe.method_8117().get(s);
			if (ingredient != class_1856.field_9017 && ingredient.method_8093(sourceStack)) {
				if (stackInSlot.method_7960()) {
					possibleSlots.add(s);
				} else if (stackInSlot.method_7909() == sourceStack.method_7909()) {
					possibleSlots.add(s);
				}
			}
		}

		if(!possibleSlots.isEmpty()){
			int totalItems =  possibleSlots.stream()
				.mapToInt(value -> inventory.method_5438(value).method_7947()).sum();
			int slots = possibleSlots.size();

			//This makes an array of ints with the best possible slot EnvTyperibution
			int[] split = new int[slots];
			int remainder = totalItems % slots;
			Arrays.fill(split, totalItems / slots);
			while (remainder > 0){
				for (int i = 0; i < split.length; i++) {
					if(remainder > 0){
						split[i] +=1;
						remainder --;
					}
				}
			}

			List<Integer> slotEnvTyperubution = possibleSlots.stream()
				.mapToInt(value -> inventory.method_5438(value).method_7947())
				.boxed().collect(Collectors.toList());

			boolean needsBalance = false;
			for (int i = 0; i < split.length; i++) {
				int required = split[i];
				if(slotEnvTyperubution.contains(required)){
					//We need to remove the int, not at the int, this seems to work around that
					slotEnvTyperubution.remove(new Integer(required));
				} else {
					needsBalance = true;
				}
			}
			if (!needsBalance) {
				return Optional.empty();
			}
		} else {
			return Optional.empty();
		}

		//Slot, count
		Pair<Integer, Integer> bestSlot = null;
		for (Integer slot : possibleSlots) {
			class_1799 slotStack = inventory.method_5438(slot);
			if (slotStack.method_7960()) {
				bestSlot = Pair.of(slot, 0);
			}
			if (bestSlot == null) {
				bestSlot = Pair.of(slot, slotStack.method_7947());
			} else if (bestSlot.getRight() >= slotStack.method_7947()) {
				bestSlot = Pair.of(slot, slotStack.method_7947());
			}
		}
		if (bestSlot == null
			|| bestSlot.getLeft() == balanceSlot
			|| bestSlot.getRight() == sourceStack.method_7947()
			|| inventory.method_5438(bestSlot.getLeft()).method_7960()
			|| !ItemUtils.isItemEqual(sourceStack, inventory.method_5438(bestSlot.getLeft()), true, true)) {
			return Optional.empty();
		}
		sourceStack.method_7934(1);
		inventory.method_5438(bestSlot.getLeft()).method_7933(1);
		inventory.setChanged();

		return Optional.of(getCraftingMatrix());
	}

	private class_1715 getCraftingMatrix() {
		if (craftCache == null) {
			craftCache = new class_1715(new RollingBEContainer(), 3, 3);
		}
		if (inventory.hasChanged()) {
			for (int i = 0; i < 9; i++) {
				craftCache.method_5447(i, inventory.method_5438(i).method_7972());
			}
			inventory.resetChanged();
		}
		return craftCache;
	}

	public boolean canMake(class_1715 craftMatrix) {
		class_1799 stack = findMatchingRecipeOutput(craftMatrix, this.field_11863);
		if (locked) {
			for (int i = 0; i < craftMatrix.method_5439(); i++) {
				class_1799 stack1 = craftMatrix.method_5438(i);
				if (!stack1.method_7960() && stack1.method_7947() < 2) {
					return false;
				}
			}
		}
		if (stack.method_7960()) {
			return false;
		}
		class_1799 output = inventory.method_5438(outputSlot);
		if (output.method_7960()) {
			return true;
		}
		return ItemUtils.isItemEqual(stack, output, true, true);
	}

	public List<RollingMachineRecipe> getAllRecipe(class_1937 world){
		return ModRecipes.ROLLING_MACHINE.getRecipes(world);
	}

	public class_1799 findMatchingRecipeOutput(class_1715 inv, class_1937 world) {
		RollingMachineRecipe recipe = findMatchingRecipe(inv, world);
		if(recipe == null){
			return class_1799.field_8037;
		}
		return recipe.method_8110();
	}

	public RollingMachineRecipe findMatchingRecipe(class_1715 inv, class_1937 world) {
		for (RollingMachineRecipe recipe : getAllRecipe(world)) {
			if (recipe.method_8115(inv, world)) {
				return recipe;
			}
		}
		return null;
	}
	
	@Override
	public class_1799 getToolDrop(final class_1657 entityPlayer) {
		return TRContent.Machine.ROLLING_MACHINE.getStack();
	}

	@Override
	public void method_11014(final class_2487 tagCompound) {
		super.method_11014(tagCompound);
		this.isRunning = tagCompound.method_10577("isRunning");
		this.tickTime = tagCompound.method_10550("tickTime");
		this.locked = tagCompound.method_10577("locked");
	}

	@Override
	public class_2487 method_11007(final class_2487 tagCompound) {
		super.method_11007(tagCompound);
		tagCompound.method_10556("isRunning", this.isRunning);
		tagCompound.method_10569("tickTime", this.tickTime);
		tagCompound.method_10556("locked", locked);
		return tagCompound;
	}

	@Override
	public RebornInventory<RollingMachineBlockEntity> getInventory() {
		return inventory;
	}

	public int getBurnTime() {
		return tickTime;
	}

	public void setBurnTime(final int burnTime) {
		this.tickTime = burnTime;
	}

	public int getBurnTimeRemainingScaled(final int scale) {
		if (tickTime == 0 || Math.max((int) (TechRebornConfig.rollingMachineRunTime* (1.0 - getSpeedMultiplier())), 1) == 0) {
			return 0;
		}
		return tickTime * scale / Math.max((int) (TechRebornConfig.rollingMachineRunTime* (1.0 - getSpeedMultiplier())), 1);
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("rollingmachine").player(player.field_7514)
			.inventory().hotbar()
			.addInventory().blockEntity(this)
			.slot(0, 30, 22).slot(1, 48, 22).slot(2, 66, 22)
			.slot(3, 30, 40).slot(4, 48, 40).slot(5, 66, 40)
			.slot(6, 30, 58).slot(7, 48, 58).slot(8, 66, 58)
			.onCraft(inv -> this.inventory.method_5447(1, findMatchingRecipeOutput(getCraftingMatrix(), this.field_11863)))
			.outputSlot(9, 124, 40)
			.energySlot(10, 8, 70)
			.syncEnergyValue().sync(this::getBurnTime, this::setBurnTime).sync(this::getLockedInt, this::setLockedInt).addInventory().create(this, syncID);
	}

	//Easyest way to sync back to the client
	public int getLockedInt() {
		return locked ? 1 : 0;
	}

	public void setLockedInt(int lockedInt) {
		locked = lockedInt == 1;
	}

	public int getProgressScaled(final int scale) {
		if (tickTime != 0 && Math.max((int) (TechRebornConfig.rollingMachineRunTime* (1.0 - getSpeedMultiplier())), 1) != 0) {
			return tickTime * scale / Math.max((int) (TechRebornConfig.rollingMachineRunTime* (1.0 - getSpeedMultiplier())), 1);
		}
		return 0;
	}

	private static class RollingBEContainer extends class_1703 {

		protected RollingBEContainer() {
			super(null, 0);
		}

		@Override
		public boolean method_7597(final class_1657 entityplayer) {
			return true;
		}

	}

	@Override
	public boolean canBeUpgraded() {
		return true;
	}
}
