/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.storage.energy.idsu;

import reborncore.common.util.NBTSerializable;
import reborncore.common.world.DataAttachment;
import reborncore.common.world.DataAttachmentProvider;

import javax.annotation.Nonnull;
import net.minecraft.class_1937;
import net.minecraft.class_2487;
import java.util.HashMap;

public class IDSUManager implements DataAttachment {

	@Nonnull
	public static IDSUPlayer getPlayer(class_1937 world, String uuid){
		return get(world).getPlayer(uuid);
	}

	public static IDSUManager get(class_1937 world){
		return DataAttachmentProvider.get(world, IDSUManager.class);
	}

	private final HashMap<String, IDSUPlayer> playerHashMap = new HashMap<>();

	@Nonnull
	public IDSUPlayer getPlayer(String uuid){
		return playerHashMap.computeIfAbsent(uuid, s -> new IDSUPlayer());
	}

	@Override
	public void read(@Nonnull class_2487 tag) {
		for(String uuid : tag.method_10541()){
			playerHashMap.put(uuid, new IDSUPlayer(tag.method_10562(uuid)));
		}
	}

	@Nonnull
	@Override
	public class_2487 write() {
		class_2487 tag = new class_2487();
		playerHashMap.forEach((uuid, player) -> tag.method_10566(uuid, player.write()));
		return tag;
	}

	public static class IDSUPlayer implements NBTSerializable {

		private double energy;

		private IDSUPlayer() {
		}

		private IDSUPlayer(class_2487 compoundTag){
			read(compoundTag);
		}

		@Nonnull
		@Override
		public class_2487 write() {
			class_2487 tag = new class_2487();
			tag.method_10549("energy", energy);
			return tag;
		}

		@Override
		public void read(@Nonnull class_2487 tag) {
			energy = tag.method_10574("energy");
		}

		public double getEnergy() {
			return energy;
		}

		public void setEnergy(double energy) {
			this.energy = energy;
		}
	}

}
