/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.storage.fluid;

import org.apache.commons.lang3.text.WordUtils;
import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.fluid.FluidUtil;
import reborncore.common.fluid.FluidValue;
import reborncore.common.util.RebornInventory;
import reborncore.common.util.StringUtils;
import reborncore.common.util.Tank;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;
import techreborn.utils.FluidUtils;

import javax.annotation.Nonnull;
import net.minecraft.class_124;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_2591;
import java.util.List;

public class TankUnitBaseBlockEntity extends MachineBaseBlockEntity implements InventoryProvider, IToolDrop, IListInfoProvider, IContainerProvider {
	protected Tank tank;
	protected RebornInventory<TankUnitBaseBlockEntity> inventory = new RebornInventory<>(2, "TankInventory", 64, this);

	private TRContent.TankUnit type;

	public TankUnitBaseBlockEntity() {
		super(TRBlockEntities.TANK_UNIT);
	}

	public TankUnitBaseBlockEntity(TRContent.TankUnit type) {
		super(TRBlockEntities.TANK_UNIT);
		configureEntity(type);
	}


	public TankUnitBaseBlockEntity(class_2591<?> blockEntityTypeIn, TRContent.TankUnit type) {
		super(blockEntityTypeIn);
		this.type = type;
	}

	private void configureEntity(TRContent.TankUnit type) {
		this.type = type;
		this.tank = new Tank("TankStorage", type.capacity, this);
	}

	@Override
	public void method_16896() {
		super.method_16896();

		if (field_11863.method_8608()) {
			return;
		}

		if (FluidUtils.drainContainers(tank, inventory, 0, 1)
			|| FluidUtils.fillContainers(tank, inventory, 0, 1, tank.getFluid())) {

			if (type == TRContent.TankUnit.CREATIVE) {
				if (!tank.isEmpty() && !tank.isFull()) {
					tank.setFluidAmount(FluidValue.INFINITE);
				}
			}
			syncWithAll();
		}
	}

	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	@Override
	public void method_11014(final class_2487 tagCompound) {
		super.method_11014(tagCompound);
		if (tagCompound.method_10545("unitType")) {
			this.type = TRContent.TankUnit.valueOf(tagCompound.method_10558("unitType"));
			configureEntity(type);
			tank.read(tagCompound);
		} else {
			// SAVE COMPAT
			if (tagCompound.method_10545("QuantumTankBlockEntity")) {
				this.type = TRContent.TankUnit.QUANTUM;
				Tank temp = new Tank("QuantumTankBlockEntity", type.capacity, this);
				temp.read(tagCompound);

				// Set tank name to what it should be
				tank = new Tank("TankStorage", type.capacity, this);
				tank.setFluid(null, temp.getFluidInstance().copy());
			}
		}


	}

	@Override
	public class_2487 method_11007(final class_2487 tagCompound) {
		super.method_11007(tagCompound);
		tagCompound.method_10582("unitType", this.type.name());
		tank.write(tagCompound);
		return tagCompound;
	}

	public class_1799 getDropWithNBT() {
		class_1799 dropStack = new class_1799(getBlockType(), 1);
		final class_2487 blockEntity = new class_2487();
		this.method_11007(blockEntity);
		dropStack.method_7980(new class_2487());
		dropStack.method_7948().method_10566("blockEntity", blockEntity);
		return dropStack;
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory<TankUnitBaseBlockEntity> getInventory() {
		return this.inventory;
	}

	// IListInfoProvider
	@Override
	public void addInfo(final List<class_2561> info, final boolean isReal, boolean hasData) {
		if (isReal || hasData) {
			if (!this.tank.getFluidInstance().isEmpty()) {
				info.add(new class_2585(this.tank.getFluidAmount() + StringUtils.t("techreborn.tooltip.unit.divider") + WordUtils.capitalize(FluidUtil.getFluidName(this.tank.getFluid()))));
			} else {
				info.add(new class_2588("techreborn.tooltip.unit.empty"));
			}
		}
		info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.unit.capacity") + class_124.field_1065 + this.tank.getCapacity() + " (" + this.tank.getCapacity().getRawValue() / 1000 + ")"));
	}

	// IContainerProvider
	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("tank").player(player.field_7514).inventory().hotbar()
			.addInventory().blockEntity(this).fluidSlot(0, 100, 53).outputSlot(1, 140, 53)
			.sync(tank).addInventory().create(this, syncID);
	}

	@Nonnull
	@Override
	public Tank getTank() {
		return tank;
	}

	@Deprecated
	public void setTank(Tank tank) {
		this.tank.setFluid(null, tank.getFluidInstance());
	}

	@Override
	public class_1799 getToolDrop(class_1657 playerEntity) {
		return getDropWithNBT();
	}
}
