/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.storage.item;

import reborncore.api.IListInfoProvider;
import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.util.ItemUtils;
import reborncore.common.util.RebornInventory;
import reborncore.common.util.StringUtils;
import reborncore.common.util.WorldUtils;
import techreborn.init.TRBlockEntities;
import techreborn.init.TRContent;

import java.util.List;
import net.minecraft.class_124;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_2591;
import net.minecraft.class_2680;

public class StorageUnitBaseBlockEntity extends MachineBaseBlockEntity
	implements InventoryProvider, IToolDrop, IListInfoProvider, IContainerProvider {


	// Inventory constants
	private static final int INPUT_SLOT = 0;
	private static final int OUTPUT_SLOT = 1;

	protected RebornInventory<StorageUnitBaseBlockEntity> inventory;
	private int maxCapacity;

	private boolean shouldUpdate = false;

	private class_1799 storeItemStack;

	private TRContent.StorageUnit type;

	public StorageUnitBaseBlockEntity() {
		super(TRBlockEntities.STORAGE_UNIT);
	}

	public StorageUnitBaseBlockEntity(TRContent.StorageUnit type) {
		super(TRBlockEntities.STORAGE_UNIT);
		configureEntity(type);
	}

	@Deprecated
	public StorageUnitBaseBlockEntity(class_2591<?> blockEntityTypeIn, String name, int maxCapacity) {
		super(blockEntityTypeIn);
		this.maxCapacity = maxCapacity;
		storeItemStack = class_1799.field_8037;
		type = TRContent.StorageUnit.QUANTUM;
		inventory = new RebornInventory<>(3, name, maxCapacity, this);
	}

	private void configureEntity(TRContent.StorageUnit type) {
		this.maxCapacity = type.capacity;
		storeItemStack = class_1799.field_8037;
		inventory = new RebornInventory<>(2, "ItemInventory", 64, this);

		this.type = type;
	}

	// TileMachineBase
	@Override
	public void method_16896() {
		super.method_16896();
		if (field_11863.field_9236) {
			return;
		}


		// If there is an item in the input AND stored is less than max capacity
		if (!inventory.method_5438(INPUT_SLOT).method_7960() && !isFull()) {
			inventory.method_5447(INPUT_SLOT, processInput(inventory.method_5438(INPUT_SLOT)));

			shouldUpdate = true;
		}

		// Fill output slot with goodies when stored has items and output count is less than max stack size
		if (storeItemStack.method_7947() > 0 && inventory.method_5438(OUTPUT_SLOT).method_7947() < getStoredStack().method_7914()) {
			populateOutput();

			shouldUpdate = true;
		}

		if (type == TRContent.StorageUnit.CREATIVE) {
			if (!isFull() && !isEmpty()) {
				fillToCapacity();
				shouldUpdate = true;
			}
		}


		if (shouldUpdate) {
			inventory.setChanged();
			method_5431();
			syncWithAll();

			shouldUpdate = false;
		}
	}

	private void populateOutput() {
		// Set to storeItemStack to get the stack type
		class_1799 output = storeItemStack.method_7972();

		int outputSlotCount = inventory.method_5438(OUTPUT_SLOT).method_7947();

		// Set to current outputSlot count
		output.method_7939(outputSlotCount);

		// Calculate amount needed to fill stack in output slot
		int amountToFill = getStoredStack().method_7914() - outputSlotCount;

		if (storeItemStack.method_7947() >= amountToFill) {
			storeItemStack.method_7934(amountToFill);

			if (storeItemStack.method_7960()) {
				storeItemStack = class_1799.field_8037;
			}

			output.method_7933(amountToFill);
		} else {
			output.method_7933(storeItemStack.method_7947());
			storeItemStack = class_1799.field_8037;
		}

		inventory.method_5447(OUTPUT_SLOT, output);
	}

	private void addStoredItemCount(int amount) {
		storeItemStack.method_7933(amount);
	}

	public class_1799 getStoredStack() {
		return storeItemStack.method_7960() ? inventory.method_5438(OUTPUT_SLOT) : storeItemStack;
	}

	public class_1799 getAll() {
		class_1799 returnStack = class_1799.field_8037;

		if (!isEmpty()) {
			returnStack = getStoredStack().method_7972();
			returnStack.method_7939(getCurrentCapacity());
		}

		return returnStack;
	}

	public void setStoredStack(class_1799 itemStack) {
		storeItemStack = itemStack;
	}

	public class_1799 processInput(class_1799 inputStack) {

		boolean isSameStack = isSameType(inputStack);

		if (storeItemStack == class_1799.field_8037 && (isSameStack || getCurrentCapacity() == 0)) {
			// Check if storage is empty, NOT including the output slot
			storeItemStack = inputStack.method_7972();

			if (inputStack.method_7947() <= maxCapacity) {
				inputStack = class_1799.field_8037;
			} else {
				// Stack is higher than capacity
				storeItemStack.method_7939(maxCapacity);
				inputStack.method_7934(maxCapacity);
			}
		} else if (isSameStack) {
			// Not empty but same type

			// Amount of items that can be added before reaching capacity
			int reminder = maxCapacity - getCurrentCapacity();


			if (inputStack.method_7947() <= reminder) {
				// Add full stack
				addStoredItemCount(inputStack.method_7947());
				inputStack = class_1799.field_8037;
			} else {
				// Add only what is needed to reach max capacity
				addStoredItemCount(reminder);
				inputStack.method_7934(reminder);
			}
		}

		return inputStack;
	}

	public boolean isSameType(class_1799 inputStack) {
		if (inputStack != class_1799.field_8037) {
			return ItemUtils.isItemEqual(getStoredStack(), inputStack, true, true);
		}
		return false;
	}

	// Creative function
	private void fillToCapacity() {
		storeItemStack = getStoredStack();
		storeItemStack.method_7939(maxCapacity);

		inventory.method_5447(OUTPUT_SLOT, class_1799.field_8037);
	}

	public boolean isFull() {
		return getCurrentCapacity() == maxCapacity;
	}

	public boolean isEmpty() {
		return getCurrentCapacity() == 0;
	}

	public int getCurrentCapacity() {
		return storeItemStack.method_7947() + inventory.method_5438(OUTPUT_SLOT).method_7947();
	}

	public int getMaxCapacity() {
		return maxCapacity;
	}

	// Other stuff


	@Override
	public boolean canBeUpgraded() {
		return false;
	}

	@Override
	public void method_11014(class_2487 tagCompound) {
		super.method_11014(tagCompound);

		if (tagCompound.method_10545("unitType")) {
			this.type = TRContent.StorageUnit.valueOf(tagCompound.method_10558("unitType"));
			configureEntity(type);
		} else {
			this.type = TRContent.StorageUnit.QUANTUM;
		}

		storeItemStack = class_1799.field_8037;

		if (tagCompound.method_10545("storedStack")) {
			storeItemStack = class_1799.method_7915(tagCompound.method_10562("storedStack"));
		}

		if (!storeItemStack.method_7960()) {
			storeItemStack.method_7939(Math.min(tagCompound.method_10550("storedQuantity"), this.maxCapacity));
		}

		inventory.read(tagCompound);
	}

	@Override
	public class_2487 method_11007(class_2487 tagCompound) {
		super.method_11007(tagCompound);

		tagCompound.method_10582("unitType", this.type.name());

		if (!storeItemStack.method_7960()) {
			class_1799 temp = storeItemStack.method_7972();
			if (storeItemStack.method_7947() > storeItemStack.method_7914()) {
				temp.method_7939(storeItemStack.method_7914());
			}
			tagCompound.method_10566("storedStack", temp.method_7953(new class_2487()));
			tagCompound.method_10569("storedQuantity", Math.min(storeItemStack.method_7947(), maxCapacity));
		} else {
			tagCompound.method_10569("storedQuantity", 0);
		}
		inventory.write(tagCompound);
		return tagCompound;
	}

	// ItemHandlerProvider
	@Override
	public RebornInventory<StorageUnitBaseBlockEntity> getInventory() {
		return inventory;
	}

	// IToolDrop
	@Override
	public class_1799 getToolDrop(class_1657 entityPlayer) {
		return getDropWithNBT();
	}

	public class_1799 getDropWithNBT() {
		class_1799 dropStack = new class_1799(getBlockType(), 1);
		final class_2487 blockEntity = new class_2487();

		this.method_11007(blockEntity);
		dropStack.method_7980(new class_2487());
		dropStack.method_7969().method_10566("blockEntity", blockEntity);

		return dropStack;
	}

	// IListInfoProvider
	@Override
	public void addInfo(final List<class_2561> info, final boolean isReal, boolean hasData) {
		if (isReal || hasData) {
			if (!this.isEmpty()) {
				info.add(new class_2585(this.getCurrentCapacity() + StringUtils.t("techreborn.tooltip.unit.divider") + this.getStoredStack().method_7964().method_10851()));
			} else {
				info.add(new class_2588("techreborn.tooltip.unit.empty"));
			}
		}

		info.add(new class_2585(class_124.field_1080 + StringUtils.t("techreborn.tooltip.unit.capacity") + class_124.field_1065 + this.getMaxCapacity() +
			" items (" + this.getMaxCapacity() / 64 + ")"));
	}

	@Override
	public void onBreak(class_1937 world, class_1657 playerEntity, class_2338 blockPos, class_2680 blockState) {
		super.onBreak(world, playerEntity, blockPos, blockState);

		// No need to drop anything for creative peeps
		if (type == TRContent.StorageUnit.CREATIVE) {
			this.inventory.method_5448();
			return;
		}

		if (storeItemStack != class_1799.field_8037) {
			if (storeItemStack.method_7914() == 64) {
				// Drop stacks (In one clump, reduce lag)
				WorldUtils.dropItem(storeItemStack, world, field_11867);
			} else {
				int size = storeItemStack.method_7914();

				for (int i = 0; i < storeItemStack.method_7947() / size; i++) {
					class_1799 toDrop = storeItemStack.method_7972();
					toDrop.method_7939(size);
					WorldUtils.dropItem(toDrop, world, field_11867);
				}

				if (storeItemStack.method_7947() % size != 0) {
					class_1799 toDrop = storeItemStack.method_7972();
					toDrop.method_7939(storeItemStack.method_7947() % size);
					WorldUtils.dropItem(toDrop, world, field_11867);
				}

			}
		}

		// Inventory gets dropped automatically
	}

	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("chest").player(player.field_7514).inventory().hotbar().addInventory()
			.blockEntity(this).slot(0, 100, 53).outputSlot(1, 140, 53).addInventory().create(this, syncID);
	}
}
