/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blocks.cable;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import net.minecraft.class_156;
import net.minecraft.class_2248;
import net.minecraft.class_2350;
import net.minecraft.class_259;
import net.minecraft.class_265;
import net.minecraft.class_2680;

public final class CableShapeUtil {

	private final CableBlock cableBlock;
	private final HashMap<class_2680, class_265> shapes;

	public CableShapeUtil(CableBlock cableBlock) {
		this.cableBlock = cableBlock;
		this.shapes = createStateShapeMap();
	}

	private HashMap<class_2680, class_265> createStateShapeMap() {
		return class_156.method_654(new HashMap<>(), map -> cableBlock.method_9595().method_11662()
				.forEach(state -> map.put(state, getStateShape(state)))
		);
	}

	private class_265 getStateShape(class_2680 state) {
		final double size = cableBlock.type != null ?  cableBlock.type.cableThickness : 6;
		final class_265 baseShape = class_2248.method_9541(size, size, size, 16.0D - size, 16.0D - size, 16.0D - size);

		final List<class_265> connections = new ArrayList<>();
		for(class_2350 dir : class_2350.values()){
			if(state.method_11654(CableBlock.PROPERTY_MAP.get(dir))) {
				double x = dir == class_2350.field_11039 ? 0 : dir == class_2350.field_11034 ? 16D : size;
				double z = dir == class_2350.field_11043 ? 0 : dir == class_2350.field_11035 ? 16D : size;
				double y = dir == class_2350.field_11033 ? 0 : dir == class_2350.field_11036 ? 16D : size;

				class_265 shape = class_2248.method_9541(x, y, z, 16.0D - size, 16.0D - size, 16.0D - size);
				connections.add(shape);
			}
		}
		return class_259.method_17786(baseShape, connections.toArray(new class_265[]{}));
	}

	public class_265 getShape(class_2680 state) {
		return shapes.get(state);
	}

}
