/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.init;

import net.fabricmc.fabric.api.loot.v1.FabricLootPoolBuilder;
import net.fabricmc.fabric.api.loot.v1.event.LootTableLoadingCallback;
import net.minecraft.class_141;
import net.minecraft.class_1935;
import net.minecraft.class_55;
import net.minecraft.class_61;
import net.minecraft.class_77;
import net.minecraft.class_79;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent.Ingots;
import techreborn.init.TRContent.Parts;

public class ModLoot {

	public static void init() {

		class_79 copperIngot = makeEntry(Ingots.COPPER);
		class_79 tinIngot = makeEntry(Ingots.TIN);
		class_79 leadIngot = makeEntry(Ingots.LEAD);
		class_79 silverIngot = makeEntry(Ingots.SILVER);
		class_79 refinedronIngot = makeEntry(Ingots.REFINED_IRON);
		class_79 advancedalloyIngot = makeEntry(Ingots.ADVANCED_ALLOY);
		class_79 basicFrame = makeEntry(TRContent.MachineBlocks.BASIC.frame.method_8389());
		class_79 basicCircuit = makeEntry(Parts.ELECTRONIC_CIRCUIT);
		
		class_79 aluminumIngot = makeEntry(Ingots.ALUMINUM);
		class_79 electrumIngot = makeEntry(Ingots.ELECTRUM);
		class_79 invarIngot = makeEntry(Ingots.INVAR);
		class_79 nickelIngot = makeEntry(Ingots.NICKEL);
		class_79 steelIngot = makeEntry(Ingots.STEEL);
		class_79 zincIngot = makeEntry(Ingots.ZINC);
		class_79 advancedFrame = makeEntry(TRContent.MachineBlocks.ADVANCED.frame.method_8389());
		class_79 advancedCircuit = makeEntry(Parts.ADVANCED_CIRCUIT);
		class_79 dataStorageChip = makeEntry(Parts.DATA_STORAGE_CHIP);
		
		class_79 chromeIngot = makeEntry(Ingots.CHROME);		
		class_79 iridiumIngot = makeEntry(Ingots.IRIDIUM);		
		class_79 platinumIngot = makeEntry(Ingots.PLATINUM);
		class_79 titaniumIngot = makeEntry(Ingots.TITANIUM);
		class_79 tungstenIngot = makeEntry(Ingots.TUNGSTEN);
		class_79 tungstensteelIngot = makeEntry(Ingots.TUNGSTENSTEEL);
		class_79 industrialFrame = makeEntry(TRContent.MachineBlocks.INDUSTRIAL.frame.method_8389());
		class_79 industrialCircuit = makeEntry(Parts.INDUSTRIAL_CIRCUIT);
		class_79 energyFlowChip = makeEntry(Parts.ENERGY_FLOW_CHIP);
		
		

		class_55 poolBasic = FabricLootPoolBuilder.builder().withEntry(copperIngot).withEntry(tinIngot)
				.withEntry(leadIngot).withEntry(silverIngot).withEntry(refinedronIngot).withEntry(advancedalloyIngot)
				.withEntry(basicFrame).withEntry(basicCircuit).withRolls(class_61.method_377(1.0f, 2.0f))
				.method_355();
		
		class_55 poolAdvanced = FabricLootPoolBuilder.builder().withEntry(aluminumIngot).withEntry(electrumIngot)
				.withEntry(invarIngot).withEntry(nickelIngot).withEntry(steelIngot).withEntry(zincIngot)
				.withEntry(advancedFrame).withEntry(advancedCircuit).withEntry(dataStorageChip).withRolls(class_61.method_377(1.0f, 3.0f))
				.method_355();

		class_55 poolIndustrial = FabricLootPoolBuilder.builder().withEntry(chromeIngot).withEntry(iridiumIngot)
				.withEntry(platinumIngot).withEntry(titaniumIngot).withEntry(tungstenIngot).withEntry(tungstensteelIngot)
				.withEntry(industrialFrame).withEntry(industrialCircuit).withEntry(energyFlowChip).withRolls(class_61.method_377(1.0f, 3.0f))
				.method_355();
		
		LootTableLoadingCallback.EVENT.register((resourceManager, lootManager, ident, supplier, setter) -> {
			String stringId = ident.toString();
			if (!stringId.startsWith("minecraft:chests")) {
				return;
			}

			if (TechRebornConfig.enableOverworldLoot) {
				switch (stringId) {
				case "minecraft:chests/abandoned_mineshaft":
                    case "minecraft:chests/desert_pyramid":
                    case "minecraft:chests/igloo_chest":
                    case "minecraft:chests/jungle_temple":
                    case "minecraft:chests/simple_dungeon":
                    case "minecraft:chests/village/village_weaponsmith":
                    case "minecraft:chests/village/village_armorer":
                    case "minecraft:chests/village/village_toolsmith":
                        supplier.withPool(poolBasic);
					break;
                    case "minecraft:chests/stronghold_corridor":
                    case "minecraft:chests/stronghold_crossing":
                    case "minecraft:chests/stronghold_library":
                        supplier.withPool(poolAdvanced);
					break;
                    case "minecraft:chests/woodland_mansion":
					 supplier.withPool(poolIndustrial);
					break;
				}
			}

			if (TechRebornConfig.enableNetherLoot) {
				if (stringId.equals("minecraft:chests/nether_bridge")) {
					supplier.withPool(poolAdvanced);
				}
			}

			if (TechRebornConfig.enableEndLoot) {
				if (stringId.equals("minecraft:chests/end_city_treasure")) {
					supplier.withPool(poolIndustrial);
				}
			}

		});
	}

	/**
	 * Makes loot entry from item provided
	 * 
	 * @param item Item to include into LootEntry
	 * @return LootEntry for item provided
	 */
	private static class_79 makeEntry(class_1935 item) {

		return class_77.method_411(item).method_437(5)
				.method_438(class_141.method_621(class_61.method_377(1.0f, 2.0f))).method_419();
	}


}
