/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items;

import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.text.WordUtils;
import reborncore.common.fluid.FluidUtil;
import reborncore.common.fluid.container.ItemFluidInfo;
import reborncore.common.util.ItemNBTHelper;
import techreborn.TechReborn;
import techreborn.init.TRContent;
import techreborn.utils.FluidUtils;

import javax.annotation.Nullable;
import net.minecraft.class_1268;
import net.minecraft.class_1271;
import net.minecraft.class_1657;
import net.minecraft.class_1761;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2263;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2371;
import net.minecraft.class_2378;
import net.minecraft.class_239;
import net.minecraft.class_2398;
import net.minecraft.class_2487;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2680;
import net.minecraft.class_2960;
import net.minecraft.class_3414;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_3486;
import net.minecraft.class_3611;
import net.minecraft.class_3612;
import net.minecraft.class_3959;
import net.minecraft.class_3965;

/**
 * Created by modmuss50 on 17/05/2016.
 */
public class ItemDynamicCell extends class_1792 implements ItemFluidInfo {

	public ItemDynamicCell() {
		super( new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP).method_7889(16));
	}

	@Override
	public void method_7850(class_1761 tab, class_2371<class_1799> subItems) {
		if (!method_7877(tab)) {
			return;
		}
		subItems.add(getEmptyCell(1));
		for (class_3611 fluid : FluidUtils.getAllFluids()) {
			if(fluid.method_15793(fluid.method_15785())){
				subItems.add(getCellWithFluid(fluid));
			}
		}
	}

	@Override
	public class_2561 method_7864(class_1799 itemStack) {
		class_3611 fluid = getFluid(itemStack);
		if(fluid != class_3612.field_15906){
			//TODO use translation keys for fluid and the cell https://fabric.asie.pl/wiki/tutorial:lang?s[]=translation might be useful
			return new class_2585(WordUtils.capitalizeFully(FluidUtil.getFluidName(fluid).replaceAll("_", " ")) + " Cell");
		}
		return super.method_7864(itemStack);
	}



	public static class_1799 getCellWithFluid(class_3611 fluid, int stackSize) {
		Validate.notNull(fluid);
		class_1799 stack = new class_1799(TRContent.CELL);
		ItemNBTHelper.getNBT(stack).method_10582("fluid", class_2378.field_11154.method_10221(fluid).toString());
		stack.method_7939(stackSize);
		return stack;
	}

	public static class_1799 getEmptyCell(int amount) {
		return new class_1799(TRContent.CELL, amount);
	}

	public static class_1799 getCellWithFluid(class_3611 fluid) {
		return getCellWithFluid(fluid, 1);
	}

	@Override
	public class_1799 getEmpty() {
		return new class_1799(this);
	}

	@Override
	public class_1799 getFull(class_3611 fluid) {
		return getCellWithFluid(fluid);
	}

	@Override
	public class_3611 getFluid(class_1799 itemStack) {
		class_2487 tag = itemStack.method_7969();
		if(tag != null && tag.method_10545("fluid")){
			return class_2378.field_11154.method_10223(new class_2960(tag.method_10558("fluid")));
		}
		return class_3612.field_15906;
	}

	@Override
	public class_1271<class_1799> method_7836(class_1937 world, class_1657 player, class_1268 hand) {
		class_1799 stack = player.method_5998(hand);
		class_3611 containedFluid = getFluid(stack);

		class_239 hitResult = method_7872(world, player, containedFluid == class_3612.field_15906 ? class_3959.class_242.field_1345 : class_3959.class_242.field_1348);
		if (hitResult.method_17783() == class_239.class_240.field_1333) {
			return class_1271.method_22430(stack);
		} else if (hitResult.method_17783() != class_239.class_240.field_1332) {
			return class_1271.method_22430(stack);
		} else {
			class_3965 blockHitResult = (class_3965)hitResult;
			class_2338 hitPos = blockHitResult.method_17777();
			class_2680 hitState = world.method_8320(hitPos);

			class_2350 side = blockHitResult.method_17780();
			class_2338 placePos = hitPos.method_10093(side);

			if (world.method_8505(player, hitPos) && player.method_7343(placePos, side, stack)) {
				if(containedFluid == class_3612.field_15906){
					if (hitState.method_11614() instanceof class_2263) {
						class_3611 drainFluid = ((class_2263)hitState.method_11614()).method_9700(world, hitPos, hitState);
						if(drainFluid != class_3612.field_15906){
							if (stack.method_7947() == 1) {
								stack = getCellWithFluid(drainFluid, 1);
							} else {
								stack.method_7934(1);
								insertOrDropStack(player, getCellWithFluid(drainFluid, 1));
							}

							playEmptyingSound(player, world, hitPos, drainFluid);
							return class_1271.method_22427(stack);
						}
					}

				} else {
					class_2680 placeState = world.method_8320(placePos);
					if(placeState.method_22360(containedFluid)){
						if (world.field_9247.method_12465() && containedFluid.method_15791(class_3486.field_15517)) {
							world.method_8396(player, placePos, class_3417.field_15102, class_3419.field_15245, 0.5F, 2.6F + (world.field_9229.nextFloat() - world.field_9229.nextFloat()) * 0.8F);
							for(int l = 0; l < 8; ++l) {
								world.method_8406(class_2398.field_11237, (double)placePos.method_10263() + Math.random(), (double)placePos.method_10264() + Math.random(), (double)placePos.method_10260() + Math.random(), 0.0D, 0.0D, 0.0D);
							}
						} else {
							world.method_8501(placePos, containedFluid.method_15785().method_15759());
							playEmptyingSound(player, world, placePos, containedFluid);
						}

						if(stack.method_7947() == 1) {
							stack = getEmpty();
						} else {
							stack.method_7934(1);
							insertOrDropStack(player, getEmpty());
						}

						return class_1271.method_22427(stack);
					}
				}
			}
		}
		return class_1271.method_22431(stack);
	}

	private void insertOrDropStack(class_1657 playerEntity, class_1799 stack){
		if(!playerEntity.field_7514.method_7394(stack)){
			playerEntity.method_5775(stack);
		}
	}

	// Thanks vanilla :)
	private void playEmptyingSound(@Nullable class_1657 playerEntity, class_1936 world, class_2338 blockPos, class_3611 fluid) {
		class_3414 soundEvent = fluid.method_15791(class_3486.field_15518) ? class_3417.field_15010 : class_3417.field_14834;
		world.method_8396(playerEntity, blockPos, soundEvent, class_3419.field_15245, 1.0F, 1.0F);
	}
}
