/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.armor;

import com.google.common.collect.HashMultimap;
import com.google.common.collect.Multimap;
import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.class_1293;
import net.minecraft.class_1294;
import net.minecraft.class_1304;
import net.minecraft.class_1322;
import net.minecraft.class_1612;
import net.minecraft.class_1657;
import net.minecraft.class_1741;
import net.minecraft.class_1761;
import net.minecraft.class_1799;
import net.minecraft.class_2371;
import reborncore.api.items.ArmorFovHandler;
import reborncore.api.items.ArmorRemoveHandler;
import reborncore.api.items.ArmorTickable;
import reborncore.api.items.ItemStackModifiers;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.ItemUtils;
import team.reborn.energy.Energy;
import team.reborn.energy.EnergyHolder;
import team.reborn.energy.EnergySide;
import team.reborn.energy.EnergyTier;
import techreborn.utils.InitUtils;

public class ItemQuantumSuit extends ItemTRArmour implements ItemStackModifiers, ArmorTickable, ArmorRemoveHandler, ArmorFovHandler, EnergyHolder {

	public static final double ENERGY_FLY = 50;
	public static final double ENERGY_SWIM = 20;
	public static final double ENERGY_BREATHING = 20;
	public static final double ENERGY_SPRINTING = 20;

	public ItemQuantumSuit(class_1741 material, class_1304 slot) {
		super(material, slot);
	}

	@Override
	public void getAttributeModifiers(class_1304 equipmentSlot, class_1799 stack, Multimap<String, class_1322> attributes) {
		attributes.removeAll(class_1612.field_7357.method_6167());

		if (this.field_7880 == class_1304.field_6172 && equipmentSlot == class_1304.field_6172) {
			if (Energy.of(stack).getEnergy() > ENERGY_SPRINTING) {
				attributes.put(class_1612.field_7357.method_6167(), new class_1322(MODIFIERS[equipmentSlot.method_5927()],"Movement Speed", 0.15, class_1322.class_1323.field_6328));
			}
		}

		if(equipmentSlot == this.field_7880 && Energy.of(stack).getEnergy() > 0) {
			attributes.put(class_1612.field_7358.method_6167(), new class_1322(MODIFIERS[field_7880.method_5927()], "Armor modifier", 20, class_1322.class_1323.field_6328));
			attributes.put(class_1612.field_7360.method_6167(), new class_1322(MODIFIERS[field_7880.method_5927()], "Knockback modifier", 2, class_1322.class_1323.field_6328));
		}
	}

	@Override
	public void tickArmor(class_1799 stack, class_1657 playerEntity) {
		switch (this.field_7880) {
			case field_6169:
				if (playerEntity.method_5869()) {
					if (Energy.of(stack).use(ENERGY_BREATHING)) {
						playerEntity.method_6092(new class_1293(class_1294.field_5923, 5, 1));
					}
				}
				break;
			case field_6174:
				if (Energy.of(stack).getEnergy() > ENERGY_FLY) {
					playerEntity.field_7503.field_7478 = true;
					if (playerEntity.field_7503.field_7479) {
						Energy.of(stack).use(ENERGY_FLY);
					}
				} else {
					playerEntity.field_7503.field_7478 = false;
					playerEntity.field_7503.field_7479 = false;
				}
				break;
			case field_6172:
				if (playerEntity.method_5624()) {
					Energy.of(stack).use(ENERGY_SPRINTING);
				}
				break;
			case field_6166:
				if (playerEntity.method_5681()) {
					if (Energy.of(stack).use(ENERGY_SWIM)) {
						playerEntity.method_6092(new class_1293(class_1294.field_5900, 5, 1));
					}
				}
				break;
			default:

		}
	}

	@Override
	public Multimap<String, class_1322> method_7844(class_1304 slot) {
		return HashMultimap.create();
	}

	@Override
	public void onRemoved(class_1657 playerEntity) {
		if (this.field_7880 == class_1304.field_6174) {
			if (!playerEntity.method_7337() && !playerEntity.method_7325()) {
				playerEntity.field_7503.field_7478 = false;
				playerEntity.field_7503.field_7479 = false;
			}
		}
	}

	@Override
	public float changeFov(float old, class_1799 stack, class_1657 playerEntity) {
		if (this.field_7880 == class_1304.field_6172 && Energy.of(stack).getEnergy() > ENERGY_SPRINTING) {
			old -= 0.6; //TODO possibly make it better
		}
		return old;
	}

	@Override
	public double getDurability(class_1799 stack) {
		return 1 - ItemUtils.getPowerForDurabilityBar(stack);
	}

	@Override
	public boolean showDurability(class_1799 stack) {
		return true;
	}

	@Override
	public int getDurabilityColor(class_1799 stack) {
		return PowerSystem.getDisplayPower().colour;
	}

	@Override
	public boolean method_7878(class_1799 itemStack_1, class_1799 itemStack_2) {
		return false;
	}

	@Override
	public double getMaxStoredPower() {
		return 40_000_000;
	}

	@Override
	public EnergyTier getTier() {
		return EnergyTier.HIGH;
	}

	@Override
	public double getMaxInput(EnergySide side) {
		return 2048;
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7850(class_1761 group, class_2371<class_1799> itemList) {
		if (!method_7877(group)) {
			return;
		}
		InitUtils.initPoweredItems(this, itemList);
	}
}
