/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tool;

import net.minecraft.class_1309;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1810;
import net.minecraft.class_1832;
import net.minecraft.class_1890;
import net.minecraft.class_1893;
import net.minecraft.class_1937;
import net.minecraft.class_2338;
import net.minecraft.class_2680;
import net.minecraft.item.*;
import reborncore.common.powerSystem.PowerSystem;
import reborncore.common.util.ItemDurabilityExtensions;
import reborncore.common.util.ItemUtils;
import team.reborn.energy.Energy;
import team.reborn.energy.EnergyHolder;
import team.reborn.energy.EnergySide;
import team.reborn.energy.EnergyTier;
import techreborn.TechReborn;

import java.util.Random;

public class DrillItem extends class_1810 implements EnergyHolder, ItemDurabilityExtensions {

	public int maxCharge;
	public int cost = 250;
	public float unpoweredSpeed;
	public float poweredSpeed;
	public int transferLimit = 100;

	public DrillItem(class_1832 material, int energyCapacity, float unpoweredSpeed, float efficiencyOnProperMaterial) {
		super(material, (int) material.method_8028(), unpoweredSpeed, new class_1792.class_1793().method_7892(TechReborn.ITEMGROUP).method_7889(1));
		this.maxCharge = energyCapacity;
		this.unpoweredSpeed = unpoweredSpeed;
		this.poweredSpeed = efficiencyOnProperMaterial;
	}

	// PickaxeItem
	@Override
	public float method_7865(class_1799 stack, class_2680 state) {
		if (Energy.of(stack).getEnergy() < cost) {
			return unpoweredSpeed;
		}
		if (class_1802.field_8647.method_7865(stack, state) > 1.0F
			|| class_1802.field_8876.method_7865(stack, state) > 1.0F) {
			return poweredSpeed;
		} else {
			return super.method_7865(stack, state);
		}
	}

	// ItemTool
	@Override
	public boolean method_7879(class_1799 stack, class_1937 worldIn, class_2680 blockIn, class_2338 pos, class_1309 entityLiving) {
		Random rand = new Random();
		if (rand.nextInt(class_1890.method_8225(class_1893.field_9119, stack) + 1) == 0) {
			Energy.of(stack).use(cost);
		}
		return true;
	}

	@Override
	public boolean method_7873(class_1799 itemstack, class_1309 entityliving, class_1309 entityliving1) {
		return true;
	}

	//Item
	@Override
	public boolean method_7878(class_1799 itemStack_1, class_1799 itemStack_2) {
		return false;
	}

	@Override
	public boolean method_7846() {
		return false;
	}

	// ItemDurabilityExtensions
	@Override
	public double getDurability(class_1799 stack) {
		return 1 - ItemUtils.getPowerForDurabilityBar(stack);
	}

	@Override
	public boolean showDurability(class_1799 stack) {
		return true;
	}

	@Override
	public int getDurabilityColor(class_1799 stack) {
		return PowerSystem.getDisplayPower().colour;
	}

	// EnergyHolder
	@Override
	public double getMaxStoredPower() {
		return maxCharge;
	}

	@Override
	public EnergyTier getTier() {
		return EnergyTier.HIGH;
	}

	@Override
	public double getMaxInput(EnergySide side) {
		return transferLimit;
	}

	@Override
	public double getMaxOutput(EnergySide side) {
		return 0;
	}
}
