/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.items.tool.industrial;

import net.fabricmc.api.EnvType;
import net.fabricmc.api.Environment;
import net.minecraft.block.*;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1309;
import net.minecraft.class_1657;
import net.minecraft.class_1761;
import net.minecraft.class_1799;
import net.minecraft.class_1802;
import net.minecraft.class_1834;
import net.minecraft.class_1836;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2371;
import net.minecraft.class_239;
import net.minecraft.class_2431;
import net.minecraft.class_2449;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2680;
import net.minecraft.class_3614;
import net.minecraft.class_3959;
import net.minecraft.class_3965;
import net.minecraft.util.*;
import reborncore.common.util.ChatUtils;
import reborncore.common.util.ItemUtils;
import reborncore.common.util.StringUtils;
import team.reborn.energy.Energy;
import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;
import techreborn.items.tool.JackhammerItem;
import techreborn.utils.InitUtils;
import techreborn.utils.MessageIDs;
import techreborn.utils.ToolsUtil;

import javax.annotation.Nullable;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class IndustrialJackhammerItem extends JackhammerItem {

	// 4M FE max charge with 1k charge rate
	public IndustrialJackhammerItem() {
		super(class_1834.field_8930, TechRebornConfig.industrialJackhammerCharge);
		this.cost = 250;
		this.transferLimit = 1000;
	}

	// Cycle Inactive, Active 3*3 and Active 5*5
	private void switchAOE(class_1799 stack, int cost, boolean isClient, int messageId){
		ItemUtils.checkActive(stack, cost, isClient, messageId);
		if (!ItemUtils.isActive(stack)) {
			ItemUtils.switchActive(stack, cost, isClient, messageId);
			stack.method_7948().method_10556("AOE5", false);
			if (isClient) {
				ChatUtils.sendNoSpamMessages(messageId, new class_2585(
						class_124.field_1080 + StringUtils.t("techreborn.message.setTo") + " "
								+ class_124.field_1065 + "3*3"));
			}
		} else {
			if (isAOE(stack)){
				ItemUtils.switchActive(stack, cost, isClient, messageId);
				stack.method_7948().method_10556("AOE5", false);
			}
			else {
				stack.method_7948().method_10556("AOE5", true);
				if (isClient) {
					ChatUtils.sendNoSpamMessages(messageId, new class_2585(
							class_124.field_1080 + StringUtils.t("techreborn.message.setTo") + " "
									+ class_124.field_1065 + "5*5"));
				}
			}
		}
	}

	private Set<class_2338> getTargetBlocks(class_1937 worldIn, class_2338 pos, @Nullable class_1309 entityLiving) {
		Set<class_2338> targetBlocks = new HashSet<>();
		if (!(entityLiving instanceof class_1657)) {
			return new HashSet<>();
		}
		class_1657 playerIn = (class_1657) entityLiving;

		//Put a dirt block down to raytrace with to stop it raytracing past the intended block
		worldIn.method_8501(pos, class_2246.field_10566.method_9564());
		class_239 hitResult = method_7872(worldIn, playerIn, class_3959.class_242.field_1348);
		worldIn.method_8501(pos, class_2246.field_10124.method_9564());

		if(!(hitResult instanceof class_3965)){
			return Collections.emptySet();
		}
		class_2350 enumfacing = ((class_3965) hitResult).method_17780();
		int add = isAOE(entityLiving.method_6047()) ? 2 : 0;
		if (enumfacing == class_2350.field_11035 || enumfacing == class_2350.field_11043) {
			for (int i = -1 - add/2; i < 2 + add/2; i++) {
				for (int j = -1; j < 2 + add; j++) {
					class_2338 newPos = pos.method_10069(i, j, 0);
					if (shouldBreak(worldIn, pos, newPos)) {
						targetBlocks.add(newPos);
					}
				}
			}
		} else if (enumfacing == class_2350.field_11034 || enumfacing == class_2350.field_11039) {
			for (int i = -1 - add/2; i < 2 + add/2; i++) {
				for (int j = -1; j < 2 + add; j++) {
					class_2338 newPos = pos.method_10069(0, j, i);
					if (shouldBreak(worldIn, pos, newPos)) {
						targetBlocks.add(newPos);
					}
				}
			}
		} else if (enumfacing == class_2350.field_11033 || enumfacing == class_2350.field_11036) {
			class_2350 playerFacing = playerIn.method_5735();
			int minX = 0;
			int maxX = 0;
			int minZ = 0;
			int maxZ = 0;
			switch (playerFacing){
				case field_11035:
					minZ = -1;
					maxZ = 3;
					minX = -2;
					maxX = 2;
					break;
				case field_11043:
					minZ = -3;
					maxZ = 1;
					minX = -2;
					maxX = 2;
					break;
				case field_11039:
					minZ = -2;
					maxZ = 2;
					minX = -3;
					maxX = 1;
					break;
				case field_11034:
					minZ = -2;
					maxZ = 2;
					minX = -1;
					maxX = 3;
					break;
			}
			for (int x = minX; x <= maxX; x++) {
				for (int z = minZ; z <= maxZ; z++) {
					class_2338 newPos = pos.method_10069(x, 0, z);
					if (shouldBreak(worldIn, pos, newPos)) {
						targetBlocks.add(newPos);
					}
				}
			}
		}
		return targetBlocks;
	}

	private boolean shouldBreak(class_1937 worldIn, class_2338 originalPos, class_2338 pos) {
		if (originalPos.equals(pos)) {
			return false;
		}
		class_2680 blockState = worldIn.method_8320(pos);
		if (blockState.method_11620() == class_3614.field_15959) {
			return false;
		}
		if (blockState.method_11620().method_15797()) {
			return false;
		}
		if (blockState.method_11614() instanceof class_2431){
			return false;
		}
		if (blockState.method_11614() instanceof class_2449){
			return false;
		}
		return (class_1802.field_8403.method_7856(blockState));
	}

	private boolean isAOE(class_1799 stack){
		return !stack.method_7960() && stack.method_7969() != null && stack.method_7969().method_10577("AOE5");
	}

	// JackhammerItem
	@Override
	public boolean method_7879(class_1799 stack, class_1937 worldIn, class_2680 stateIn, class_2338 pos, class_1309 entityLiving) {
		if(ItemUtils.isActive(stack)){
			for (class_2338 additionalPos : getTargetBlocks(worldIn, pos, entityLiving)) {
				ToolsUtil.breakBlock(stack, worldIn, additionalPos, entityLiving, cost);
			}
		}
		return super.method_7879(stack, worldIn, stateIn, pos, entityLiving);
	}

	// PickaxeItem
	@Override
	public float method_7865(class_1799 stack, class_2680 state) {
		if (state.method_11620() == class_3614.field_15914 && Energy.of(stack).getEnergy() >= cost) {
			// x4 diamond mining speed
			return 32.0F;
		} else {
			return 0.5F;
		}
	}

	// Item
	@Override
	public class_1271<class_1799> method_7836(final class_1937 world, final class_1657 player, final class_1268 hand) {
		final class_1799 stack = player.method_5998(hand);
		if (player.method_5715()) {
			switchAOE(stack, cost, world.field_9236, MessageIDs.poweredToolID);
			return new class_1271<>(class_1269.field_5812, stack);
		}
		return new class_1271<>(class_1269.field_5811, stack);
	}

	@Override
	public void method_7852(class_1937 world, class_1309 entity, class_1799 stack, int i) {
		ItemUtils.checkActive(stack, cost, entity.field_6002.field_9236, MessageIDs.poweredToolID);
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7851(class_1799 stack, @Nullable class_1937 worldIn, List<class_2561> tooltip, class_1836 flagIn) {
		ItemUtils.buildActiveTooltip(stack, tooltip);
		if (ItemUtils.isActive(stack)){
			if (isAOE(stack)) {
				tooltip.add(new class_2585("5*5").method_10854(class_124.field_1061));
			} else {
				tooltip.add(new class_2585("3*3").method_10854(class_124.field_1061));
			}
		}
	}

	@Environment(EnvType.CLIENT)
	@Override
	public void method_7850(
		class_1761 par2ItemGroup, class_2371<class_1799> itemList) {
		if (!method_7877(par2ItemGroup)) {
			return;
		}
		InitUtils.initPoweredItems(TRContent.INDUSTRIAL_JACKHAMMER, itemList);
	}
}
