/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.utils;

import javax.annotation.Nonnull;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2680;
import net.minecraft.class_3494;
import net.minecraft.class_3503;
import net.minecraft.class_3611;

public class TagUtils {

	public static <T> boolean hasTag(T type, class_3494<T> tag) {
		return tag.method_15141(type);
	}

	public static class_3503<class_2248> getAllBlockTags(class_1937 world) {
		return world.method_8514().method_15202();
	}

	public static class_3503<class_1792> getAllItemTags(class_1937 world) {
		return world.method_8514().method_15201();
	}

	public static class_3503<class_3611> getAllFluidTags(class_1937 world) {
		return world.method_8514().method_15205();
	}

	public static String toFirstLower(String string) {
		if (string == null || string.isEmpty())
			return string;
		return Character.toLowerCase(string.charAt(0)) + string.substring(1);
	}

	public static String toFirstUpper(String string) {
		if (string.isEmpty())
			return string;
		return Character.toUpperCase(string.charAt(0)) + string.substring(1);
	}

	public static String joinDictName(String prefix, String name) {
		return prefix + toFirstUpper(name);
	}

	public static String[] getDictData(String prefixed) {
		StringBuilder prefixBuilder = new StringBuilder();
		StringBuilder nameBuilder = new StringBuilder();
		boolean prefixFinished = false;
		for (int i = 0; i < prefixed.length(); i++) {
			char charAt = prefixed.charAt(i);
			if (!prefixFinished) {
				if (Character.isUpperCase(charAt)) {
					nameBuilder.append(Character.toLowerCase(charAt));
					prefixFinished = true;
				} else
					prefixBuilder.append(charAt);
			} else
				nameBuilder.append(charAt);
		}
		return new String[] {
			prefixBuilder.toString(),
			nameBuilder.toString()
		};
	}

	public static boolean isDictPrefixed(String name, String... prefixes) {
		for (String prefix : prefixes)
			if (name.startsWith(prefix))
				return true;
		return false;
	}

	@Deprecated
	@Nonnull
	public static class_1799 getDictOreOrEmpty(String name, int amount) {
		throw new UnsupportedOperationException("Move to tags");
	}


	@Deprecated
	public static boolean isOre(class_2680 state, String oreName) {
		throw new UnsupportedOperationException("Move to tags");
	}


	@Deprecated
	public static boolean isOre(
		@Nonnull
			class_1799 stack, String oreName) {
		throw new UnsupportedOperationException("Move to tags");
	}

}
