/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2020 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.utils;

import techreborn.config.TechRebornConfig;
import techreborn.init.TRContent;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.class_1074;
import net.minecraft.class_124;
import net.minecraft.class_2561;
import net.minecraft.class_2585;
import net.minecraft.class_2588;
import net.minecraft.class_437;

public class ToolTipAssistUtils {

	// Colour constants
	private static final class_124 instructColour = class_124.field_1078;

	private static final class_124 infoColour = class_124.field_1065;
	private static final class_124 statColour = class_124.field_1065;

	private static final class_124 posColour = class_124.field_1060;
	private static final class_124 negColour = class_124.field_1061;


	public static List<class_2561> getUpgradeStats(TRContent.Upgrades upgradeType, int count, boolean shiftHeld) {
		List<class_2561> tips = new ArrayList<>();
		boolean shouldStackCalculate = true;

		switch (upgradeType) {
			case OVERCLOCKER:
				tips.add(getPositive("Speed increase", calculateValue(TechRebornConfig.overclockerSpeed * 100, count, shiftHeld), "%"));
				tips.add(getNegative("Energy increase", calculateValue(TechRebornConfig.overclockerPower * 100, count, shiftHeld), "%"));
				break;
			case TRANSFORMER:
				shouldStackCalculate = false;
				break;
			case ENERGY_STORAGE:
				tips.add(getPositive("Storage increase", calculateValue(TechRebornConfig.energyStoragePower, count, shiftHeld), " E"));
				break;
			case SUPERCONDUCTOR:
				tips.add(getPositive("Increased flow by: ", calculateValue(Math.pow(2, (TechRebornConfig.superConductorCount + 2)) * 100, count, shiftHeld), "%"));
				break;
		}

		// Add reminder that they can use shift to calculate the entire stack
		if(shouldStackCalculate && !shiftHeld){
			tips.add(new class_2585(instructColour + "Hold shift for stack calculation"));
		}

		return tips;
	}

	public static void addInfo(String inKey, List<class_2561> list){
		addInfo(inKey, list, true);
	}

	public static void addInfo(String inKey, List<class_2561> list, boolean hidden){
		String key = ("techreborn.message.info." + inKey);

		if(class_1074.method_4663(key)){
			if(!hidden || class_437.hasShiftDown()){
				String info = new class_2588(key).method_10851();
				String infoLines[] = info.split("\\r?\\n");

				for (String infoLine: infoLines) {
					list.add(new class_2585(infoColour + infoLine));
				}
			}else{
				list.add(new class_2585(instructColour + "Hold shift for info"));
			}
		}
	}


	private static int calculateValue(double value, int count, boolean shiftHeld) {
		int calculatedVal;

		if (shiftHeld) {
			calculatedVal = (int) value * count;
		} else {
			calculatedVal = (int) value;
		}

		return calculatedVal;
	}

	private static class_2561 getPositive(String text, int value, String unit) {
		return new class_2585(posColour + getStatStringUnit(text, value, unit));
	}

	private static class_2561 getNegative(String text, int value, String unit) {
		return new class_2585(negColour + getStatStringUnit(text, value, unit));
	}

	private static String getStatStringUnit(String text, int value, String unit) {
		return text + ": " + statColour + value + unit;
	}
}
